@@ -1,6 +1,6 @@
 ##
 ##
-## Copyright (c) 2009, Brandon Whitcher and Volker Schmid
+## Copyright (c) 2009,2010 Brandon Whitcher and Volker Schmid
 ## All rights reserved.
 ## 
 ## Redistribution and use in source and binary forms, with or without
@@ -32,40 +32,57 @@
 ## $Id: readS4.R 332 2010-01-29 16:54:07Z bjw34032 $
 ##
 
+## Sub-routines
+.readCharWithEmbeddedNuls <- function(fid, n, to="UTF-8") {
+  txt <- readBin(fid, "raw", n)
+  iconv(rawToChar(txt[txt != as.raw(0)]), to=to)
+}
+
+##
+##
+##
+
 readNIfTI <- function(fname, verbose=FALSE, warn=-1, reorient=TRUE,
                       call=NULL) {
   if (is.null(call)) {
     call <- match.call()
   }
   ## Warnings?
-  oldwarn <- options()$warn
+  oldwarn <- getOption("warn")
   options(warn=warn)
 
-  if (verbose)
+  if (verbose) {
     cat(paste("  fname =", fname), fill=TRUE)
+  }
   ## Strip any extensions
   fname <- sub(".gz", "", fname)
   fname <- sub(".nii", "", fname)
 
+  if (! (file.exists(paste(fname, "nii", sep=".")) ||
+         file.exists(paste(fname, "nii.gz", sep=".")))) {
+    stop("File(s) not found!")
+  }
   if (file.exists(paste(fname, "nii", sep=".")) &&
       file.exists(paste(fname, "nii.gz", sep="."))) {
-    stop("-- Both compressed and uncompressed files exist! --")
+    stop("Both compressed and uncompressed files exist!")
   } else {
     ## If uncompressed files exist, then upload!
     if (file.exists(paste(fname, "nii", sep="."))) {      
-      if (verbose)
+      if (verbose) {
         cat(paste("  files = ", fname, ".nii", sep=""), fill=TRUE)
-      nim <- read.nifti.content(fname, gzipped=FALSE, verbose=verbose,
-                                warn=warn, reorient=reorient, call=call)
+      }
+      nim <- .read.nifti.content(fname, gzipped=FALSE, verbose=verbose,
+                                 warn=warn, reorient=reorient, call=call)
       options(warn=oldwarn)
       return(nim)
     }
     ## If compressed files exist, then upload!
     if (file.exists(paste(fname, "nii.gz", sep="."))) {
-      if (verbose)
+      if (verbose) {
         cat(paste("  files = ", fname, ".nii.gz", sep=""), fill=TRUE)
-      nim <- read.nifti.content(fname, gzipped=TRUE, verbose=verbose,
-                                warn=warn, reorient=reorient, call=call)
+      }
+      nim <- .read.nifti.content(fname, gzipped=TRUE, verbose=verbose,
+                                 warn=warn, reorient=reorient, call=call)
       options(warn=oldwarn)
       return(nim)
     }
@@ -77,14 +94,9 @@ readNIfTI <- function(fname, verbose=FALSE, warn=-1, reorient=TRUE,
 ############################################################################
 ############################################################################
 
-read.nifti.content <- function(fname, onefile=TRUE, gzipped=TRUE,
-                               verbose=FALSE, warn=-1, reorient=FALSE,
-                               call=NULL) {
-  ## Function to remove embedded nuls in character string (raw format) 
-  readCharWithEmbeddedNuls <- function(fid, n) {
-    txt <- readBin(fid, "raw", n)
-    iconv(rawToChar(txt[txt != as.raw(0)]), to="UTF-8")
-  }
+.read.nifti.content <- function(fname, onefile=TRUE, gzipped=TRUE,
+                                verbose=FALSE, warn=-1, reorient=FALSE,
+                                call=NULL) {
   ## Open appropriate file
   if (gzipped) {
     suffix <- ifelse(onefile, "nii.gz", "hdr.gz")
@@ -99,7 +111,7 @@ read.nifti.content <- function(fname, onefile=TRUE, gzipped=TRUE,
     cat("  nii   =", fname, fill=TRUE)
   }
   ## Warnings?
-  oldwarn <- options()$warn
+  oldwarn <- getOption("warn")
   options(warn=warn)
   ## Test for endian properties
   endian <- .Platform$endian
@@ -120,13 +132,12 @@ read.nifti.content <- function(fname, onefile=TRUE, gzipped=TRUE,
   ## Construct S4 object
   nim <- nifti()
   nim@"sizeof_hdr" <- sizeof.hdr
-  nim@"data_type" <- readCharWithEmbeddedNuls(fid, n=10)
-  nim@"db_name" <- readCharWithEmbeddedNuls(fid, n=18)
+  nim@"data_type" <- .readCharWithEmbeddedNuls(fid, n=10)
+  nim@"db_name" <- .readCharWithEmbeddedNuls(fid, n=18)
   nim@"extents" <- readBin(fid, integer(), size=4, endian=endian)
   nim@"session_error" <- readBin(fid, integer(), size=2, endian=endian)
-  nim@"regular" <- readCharWithEmbeddedNuls(fid, n=1)
-  nim@"dim_info" <- readBin(fid, integer(), size=1, signed=FALSE,
-                            endian=endian)
+  nim@"regular" <- .readCharWithEmbeddedNuls(fid, n=1)
+  nim@"dim_info" <- .readCharWithEmbeddedNuls(fid, n=1)
   nim@"dim_" <- readBin(fid, integer(), 8, size=2, endian=endian)
   nim@"intent_p1" <- readBin(fid, numeric(), size=4, endian=endian)
   nim@"intent_p2" <- readBin(fid, numeric(), size=4, endian=endian)
@@ -153,8 +164,8 @@ read.nifti.content <- function(fname, onefile=TRUE, gzipped=TRUE,
   nim@"toffset" <- readBin(fid, numeric(), size=4, endian=endian)
   nim@"glmax" <- readBin(fid, integer(), size=4, endian=endian)
   nim@"glmin" <- readBin(fid, integer(), size=4, endian=endian)
-  nim@"descrip" <- readCharWithEmbeddedNuls(fid, n=80)
-  nim@"aux_file" <- readCharWithEmbeddedNuls(fid, n=24)
+  nim@"descrip" <- .readCharWithEmbeddedNuls(fid, n=80)
+  nim@"aux_file" <- .readCharWithEmbeddedNuls(fid, n=24)
   nim@"qform_code" <- readBin(fid, integer(), size=2, endian=endian)
   nim@"sform_code" <- readBin(fid, integer(), size=2, endian=endian)
   nim@"quatern_b" <- readBin(fid, numeric(), size=4, endian=endian)
@@ -166,8 +177,8 @@ read.nifti.content <- function(fname, onefile=TRUE, gzipped=TRUE,
   nim@"srow_x" <- readBin(fid, numeric(), 4, size=4, endian=endian)
   nim@"srow_y" <- readBin(fid, numeric(), 4, size=4, endian=endian)
   nim@"srow_z" <- readBin(fid, numeric(), 4, size=4, endian=endian)
-  nim@"intent_name" <- readCharWithEmbeddedNuls(fid, n=16)
-  nim@"magic" <- readCharWithEmbeddedNuls(fid, n=4)
+  nim@"intent_name" <- .readCharWithEmbeddedNuls(fid, n=16)
+  nim@"magic" <- .readCharWithEmbeddedNuls(fid, n=4)
   ## To flag such a struct as being conformant to the NIFTI-1 spec,
   ## the last 4 bytes of the header must be either the C String "ni1"
   ## or "n+1"; in hexadecimal, the 4 bytes [6E 69 31 00] or [6E 2B 31
@@ -204,7 +215,7 @@ read.nifti.content <- function(fname, onefile=TRUE, gzipped=TRUE,
       nimextsec <- new("niftiExtensionSection")
       nimextsec@esize <- readBin(fid, integer(), size=4, endian=endian)
       nimextsec@ecode <- readBin(fid, integer(), size=4, endian=endian)
-      nimextsec@edata <- readCharWithEmbeddedNuls(fid, n=nimextsec@esize-8)
+      nimextsec@edata <- .readCharWithEmbeddedNuls(fid, n=nimextsec@esize-8)
       nim@extensions <- append(nim@extensions, nimextsec)
     }
     if (seek(fid) > nim@"vox_offset") {
@@ -212,7 +223,9 @@ read.nifti.content <- function(fname, onefile=TRUE, gzipped=TRUE,
     }
   }
 
-  if (verbose) cat("  seek(fid) =", seek(fid), fill=TRUE) 
+  if (verbose) {
+    cat("  seek(fid) =", seek(fid), fill=TRUE)
+  }
   n <- prod(nim@"dim_"[2:5])
   if (!onefile) {
     close(fid)
@@ -237,9 +250,9 @@ read.nifti.content <- function(fname, onefile=TRUE, gzipped=TRUE,
                       fname, ".img", sep=""))
            )
   close(fid)
-
+  ##
   ## THE SLOW BIT FOLLOWS
-
+  ##
   ## 3D IMAGE (VOLUME) ORIENTATION AND LOCATION IN SPACE:
   ## There are 3 different methods by which continuous coordinates can
   ## attached to voxels.  The discussion below emphasizes 3D volumes,
@@ -284,32 +297,40 @@ read.nifti.content <- function(fname, onefile=TRUE, gzipped=TRUE,
 
 readANALYZE <- function(fname, verbose=FALSE, warn=-1) {
   ## Warnings?
-  oldwarn <- options()$warn
+  oldwarn <- getOption("warn")
   options(warn=warn)
-
-  if (verbose) cat(paste("  fname =", fname), fill=TRUE)
+  if (verbose) {
+    cat(paste("  fname =", fname), fill=TRUE)
+  }
   ## Strip any extensions
   fname <- sub(".gz", "", fname)
   fname <- sub(".hdr", "", fname)
   fname <- sub(".img", "", fname)
-
+  if (! (file.exists(paste(fname, "hdr", sep=".")) &&
+         file.exists(paste(fname, "img", sep="."))) &&
+      ! (file.exists(paste(fname, "hdr.gz", sep=".")) &&
+         file.exists(paste(fname, "img.gz", sep=".")))) {
+    stop("File(s) not found!")
+  }
   ## If uncompressed files exist, then upload!
   if (file.exists(paste(fname, "hdr", sep=".")) &&
       file.exists(paste(fname, "img", sep="."))) {      
-    if (verbose)
+    if (verbose) {
       cat(paste("  files = ", fname, ".{hdr,img}", sep=""), fill=TRUE)
-    aim <- read.analyze.content(fname, gzipped=FALSE, verbose=verbose,
-                                warn=warn)
+    }
+    aim <- .read.analyze.content(fname, gzipped=FALSE, verbose=verbose,
+                                 warn=warn)
     options(warn=oldwarn)
     return(aim)
   }
   ## If compressed files exist, then upload!
   if (file.exists(paste(fname, "hdr.gz", sep=".")) &&
       file.exists(paste(fname, "img.gz", sep="."))) {      
-    if (verbose)
+    if (verbose) {
       cat(paste("  files = ", fname, ".{hdr.gz,img.gz}", sep=""), fill=TRUE)
-    aim <- read.analyze.content(fname, gzipped=TRUE, verbose=verbose,
-                                warn=warn)
+    }
+    aim <- .read.analyze.content(fname, gzipped=TRUE, verbose=verbose,
+                                 warn=warn)
     options(warn=oldwarn)
     return(aim)
   }
@@ -320,13 +341,8 @@ readANALYZE <- function(fname, verbose=FALSE, warn=-1) {
 ############################################################################
 ############################################################################
 
-read.analyze.content <- function(fname, gzipped=TRUE, verbose=FALSE,
-                                 warn=-1) {
-  ## Function to remove embedded nuls in character string (raw format) 
-  readCharWithEmbeddedNuls <- function(fid, n) {
-    txt <- readBin(fid, "raw", n)
-    iconv(rawToChar(txt[txt != as.raw(0)]), to="UTF-8")
-  }
+.read.analyze.content <- function(fname, gzipped=TRUE, verbose=FALSE,
+                                  warn=-1) {
   ## Open header file
   if (gzipped) {
     fname <- paste(fname, "hdr.gz", sep=".")
@@ -335,9 +351,11 @@ read.analyze.content <- function(fname, gzipped=TRUE, verbose=FALSE,
     fname <- paste(fname, "hdr", sep=".")
     fid <- file(fname, "rb")
   }
-  if (verbose) cat("  hdr   =", fname, fill=TRUE)
+  if (verbose) {
+    cat("  hdr   =", fname, fill=TRUE)
+  }
   ## Warnings?
-  oldwarn <- options()$warn
+  oldwarn <- getOption("warn")
   options(warn=warn)
   ## Test for endian properties
   endian <- .Platform$endian
@@ -355,15 +373,15 @@ read.analyze.content <- function(fname, gzipped=TRUE, verbose=FALSE,
   ## Construct S4 object
   aim <- new("anlz")
   aim@"sizeof_hdr" <- sizeof.hdr
-  aim@"db_type" <- readCharWithEmbeddedNuls(fid, 10) # readChar(fid, n=10)
-  aim@"db_name" <- readCharWithEmbeddedNuls(fid, n=18)
+  aim@"db_type" <- .readCharWithEmbeddedNuls(fid, 10)
+  aim@"db_name" <- .readCharWithEmbeddedNuls(fid, n=18)
   aim@"extents" <- readBin(fid, integer(), size=4, endian=endian)
   aim@"session_error" <- readBin(fid, integer(), size=2, endian=endian)
-  aim@"regular" <- readCharWithEmbeddedNuls(fid, n=1)
-  aim@"hkey_un0" <- readCharWithEmbeddedNuls(fid, n=1)
+  aim@"regular" <- .readCharWithEmbeddedNuls(fid, n=1)
+  aim@"hkey_un0" <- .readCharWithEmbeddedNuls(fid, n=1)
   aim@"dim_" <- readBin(fid, integer(), 8, size=2, endian=endian)
-  aim@"vox_units" <- readCharWithEmbeddedNuls(fid, n=4)
-  aim@"cal_units" <- readCharWithEmbeddedNuls(fid, n=8)
+  aim@"vox_units" <- .readCharWithEmbeddedNuls(fid, n=4)
+  aim@"cal_units" <- .readCharWithEmbeddedNuls(fid, n=8)
   aim@"unused1" <- readBin(fid, integer(), size=2, endian=endian)
   aim@"datatype" <- readBin(fid, integer(), size=2, endian=endian)
   aim@"bitpix" <- readBin(fid, integer(), size=2, endian=endian)
@@ -379,16 +397,16 @@ read.analyze.content <- function(fname, gzipped=TRUE, verbose=FALSE,
   aim@"verified" <- readBin(fid, numeric(), size=4, endian=endian)
   aim@"glmax" <- readBin(fid, integer(), size=4, endian=endian)
   aim@"glmin" <- readBin(fid, integer(), size=4, endian=endian)
-  aim@"descrip" <- readCharWithEmbeddedNuls(fid, n=80)
-  aim@"aux_file" <- readCharWithEmbeddedNuls(fid, n=24)
-  aim@"orient" <- readCharWithEmbeddedNuls(fid, n=1)
-  aim@"originator" <- readCharWithEmbeddedNuls(fid, 10) # readChar(fid, n=10)
-  aim@"generated" <- readCharWithEmbeddedNuls(fid, 10) # readChar(fid, n=10)
-  aim@"scannum" <- readCharWithEmbeddedNuls(fid, 10) # readChar(fid, n=10)
-  aim@"patient_id" <- readCharWithEmbeddedNuls(fid, 10) # readChar(fid, n=10)
-  aim@"exp_date" <- readCharWithEmbeddedNuls(fid, 10) # readChar(fid, n=10)
-  aim@"exp_time" <- readCharWithEmbeddedNuls(fid, 10) # readChar(fid, n=10)
-  aim@"hist_un0" <- readCharWithEmbeddedNuls(fid, n=3)
+  aim@"descrip" <- .readCharWithEmbeddedNuls(fid, n=80)
+  aim@"aux_file" <- .readCharWithEmbeddedNuls(fid, n=24)
+  aim@"orient" <- .readCharWithEmbeddedNuls(fid, n=1)
+  aim@"originator" <- .readCharWithEmbeddedNuls(fid, 10)
+  aim@"generated" <- .readCharWithEmbeddedNuls(fid, 10)
+  aim@"scannum" <- .readCharWithEmbeddedNuls(fid, 10)
+  aim@"patient_id" <- .readCharWithEmbeddedNuls(fid, 10)
+  aim@"exp_date" <- .readCharWithEmbeddedNuls(fid, 10)
+  aim@"exp_time" <- .readCharWithEmbeddedNuls(fid, 10)
+  aim@"hist_un0" <- .readCharWithEmbeddedNuls(fid, n=3)
   aim@"views" <- readBin(fid, integer(), size=4, endian=endian)
   aim@"vols_added" <- readBin(fid, integer(), size=4, endian=endian)
   aim@"start_field" <- readBin(fid, integer(), size=4, endian=endian)
@@ -406,23 +424,29 @@ read.analyze.content <- function(fname, gzipped=TRUE, verbose=FALSE,
     fname <- sub(".hdr", ".img", fname) # paste(fname, "img", sep=".")
     fid <- file(fname, "rb")
   }
-  if (verbose) cat("  img   =", fname, fill=TRUE)
+  if (verbose) {
+    cat("  img   =", fname, fill=TRUE)
+  }
   n <- prod(aim@"dim_"[2:5])
   data <- switch(as.character(aim@"datatype"),
-                 "1" = readBin(fid, integer(), n, aim@"bitpix"/8, signed=FALSE, endian=endian),
-                 "2" = readBin(fid, integer(), n, aim@"bitpix"/8, signed=FALSE, endian=endian),
-                 "4" = readBin(fid, integer(), n, aim@"bitpix"/8, endian=endian),
-                 "8" = readBin(fid, integer(), n, aim@"bitpix"/8, endian=endian),
-                 "16" = readBin(fid, numeric(), n, aim@"bitpix"/8, endian=endian),
-                 "64" = readBin(fid, double(), n, aim@"bitpix"/8, endian=endian),
+                 "1" = readBin(fid, integer(), n, aim@"bitpix"/8,
+                   signed=FALSE, endian=endian),
+                 "2" = readBin(fid, integer(), n, aim@"bitpix"/8,
+                   signed=FALSE, endian=endian),
+                 "4" = readBin(fid, integer(), n, aim@"bitpix"/8,
+                   endian=endian),
+                 "8" = readBin(fid, integer(), n, aim@"bitpix"/8,
+                   endian=endian),
+                 "16" = readBin(fid, numeric(), n, aim@"bitpix"/8,
+                   endian=endian),
+                 "64" = readBin(fid, double(), n, aim@"bitpix"/8,
+                   endian=endian),
                  stop(paste("Data type ", aim@"datatype", " (",
                             convert.datatype.anlz(aim@"datatype"), 
                             ") unsupported in", fname, sep="")))
   close(fid)
-
   dims <- 2:(1+aim@"dim_"[1])
   aim@.Data <- array(data, aim@"dim_"[dims])
-  
   ## Warnings?
   options(warn=oldwarn)
   ## Check validity
