@@ -1,4 +1,4 @@
 mniRL <- readNIfTI(file.path(system.file("nifti", package="oro.nifti"),
-                             "/avg152T1_RL_nifti"))
+                             "mniRL_nifti"))
 image(mniRL)
 orthographic(mniRL)
